@@ -27,6 +27,7 @@ import re
 import datetime
 import time
 import gtk
+import glib
 import gobject
 import gconf
 import dbus
@@ -36,6 +37,8 @@ from sugar.graphics import style
 from sugar.graphics.palette import Palette
 from sugar.graphics.toolbutton import ToolButton
 from sugar.graphics.tray import TrayIcon
+from sugar.graphics.menuitem import MenuItem
+from sugar.graphics.icon import Icon
 from sugar.graphics import xocolor
 from sugar.util import unique_id
 from sugar import profile
@@ -63,26 +66,18 @@ _GSM_STATE_NOT_READY = 0
 _GSM_STATE_DISCONNECTED = 1
 _GSM_STATE_CONNECTING = 2
 _GSM_STATE_CONNECTED = 3
-_GSM_STATE_NEED_AUTH = 4
+_GSM_STATE_FAILED = 4
 
-def frequency_to_channel(frequency):
-    ftoc = { 2412: 1, 2417: 2, 2422: 3, 2427: 4,
-             2432: 5, 2437: 6, 2442: 7, 2447: 8,
-             2452: 9, 2457: 10, 2462: 11, 2467: 12,
-             2472: 13}
-    return ftoc[frequency]
 
 class WirelessPalette(Palette):
     __gtype_name__ = 'SugarWirelessPalette'
 
     __gsignals__ = {
         'deactivate-connection' : (gobject.SIGNAL_RUN_FIRST,
-                                   gobject.TYPE_NONE, ([])),
-        'create-connection'     : (gobject.SIGNAL_RUN_FIRST,
-                                   gobject.TYPE_NONE, ([])),
+                                   gobject.TYPE_NONE, ([]))
     }
 
-    def __init__(self, primary_text, can_create=True):
+    def __init__(self, primary_text):
         Palette.__init__(self, label=primary_text)
 
         self._disconnect_item = None
@@ -109,16 +104,12 @@ class WirelessPalette(Palette):
         self._info.pack_start(_padded(self._ip_address_label))
         self._info.show_all()
 
-        self._disconnect_item = gtk.MenuItem(_('Disconnect...'))
+        self._disconnect_item = MenuItem(_('Disconnect...'))
+        icon = Icon(icon_size=gtk.ICON_SIZE_MENU, icon_name='media-eject')
+        self._disconnect_item.set_image(icon)
         self._disconnect_item.connect('activate', self.__disconnect_activate_cb)
         self.menu.append(self._disconnect_item)
 
-        if can_create:
-            self._adhoc_item = gtk.MenuItem(_('Create new wireless network'))
-            self._adhoc_item.connect('activate', self.__adhoc_activate_cb)
-            self.menu.append(self._adhoc_item)
-            self._adhoc_item.show()
-
     def set_connecting(self):
         self.props.secondary_text = _('Connecting...')
 
@@ -145,14 +136,8 @@ class WirelessPalette(Palette):
     def __disconnect_activate_cb(self, menuitem):
         self.emit('deactivate-connection')
 
-    def __adhoc_activate_cb(self, menuitem):
-        self.emit('create-connection')
-
     def _set_frequency(self, frequency):
-        try:
-            channel = frequency_to_channel(frequency)
-        except KeyError:
-            channel = 0
+        channel = network.frequency_to_channel(frequency)
         self._set_channel(channel)
 
     def _set_channel(self, channel):
@@ -229,34 +214,53 @@ class GsmPalette(Palette):
         Palette.__init__(self, label=_('Wireless modem'))
 
         self._current_state = None
+        self._failed_connection = False
 
-        self._toggle_state_item = gtk.MenuItem('')
+        self._toggle_state_item = MenuItem('')
         self._toggle_state_item.connect('activate', self.__toggle_state_cb)
         self.menu.append(self._toggle_state_item)
         self._toggle_state_item.show()
 
-        self.set_state(_GSM_STATE_NOT_READY)
-
         self.info_box = gtk.VBox()
 
-        self.data_label = gtk.Label()
-        self.data_label.props.xalign = 0.0
-        label_alignment = self._add_widget_with_padding(self.data_label)
-        self.info_box.pack_start(label_alignment)
-        self.data_label.show()
+        self.error_title_label = gtk.Label("")
+        self.error_title_label.set_alignment(0, 0.5)
+        self.error_title_label.set_line_wrap(True)
+        self.info_box.pack_start(self.error_title_label)
+        self.error_description_label = gtk.Label("")
+        self.error_description_label.set_alignment(0, 0.5)
+        self.error_description_label.set_line_wrap(True)
+        self.info_box.pack_start(self.error_description_label)
+
+        self.connection_info_box = gtk.HBox()
+        icon = Icon(icon_name='data-upload', icon_size=gtk.ICON_SIZE_MENU)
+        self.connection_info_box.pack_start(icon)
+        icon.show()
+
+        self._data_label_up = gtk.Label()
+        self._data_label_up.props.xalign = 0.0
+        label_alignment = self._add_widget_with_padding(self._data_label_up)
+        self.connection_info_box.pack_start(label_alignment)
+        self._data_label_up.show()
         label_alignment.show()
 
-        self.connection_time_label = gtk.Label()
-        self.connection_time_label.props.xalign = 0.0
-        label_alignment = self._add_widget_with_padding( \
-                self.connection_time_label)
-        self.info_box.pack_start(label_alignment)
-        self.connection_time_label.show()
+        icon = Icon(icon_name='data-download', icon_size=gtk.ICON_SIZE_MENU)
+        self.connection_info_box.pack_start(icon)
+        icon.show()
+        self._data_label_down = gtk.Label()
+        self._data_label_down.props.xalign = 0.0
+        label_alignment = self._add_widget_with_padding(self._data_label_down)
+        self.connection_info_box.pack_start(label_alignment)
+        self._data_label_down.show()
         label_alignment.show()
 
+        self.info_box.pack_start(self.connection_info_box)
+
         self.info_box.show()
         self.set_content(self.info_box)
 
+        self.set_state(_GSM_STATE_NOT_READY)
+
     def _add_widget_with_padding(self, child, xalign=0, yalign=0.5):
         alignment = gtk.Alignment(xalign=xalign, yalign=yalign,
                                   xscale=1, yscale=0.33)
@@ -267,31 +271,41 @@ class GsmPalette(Palette):
         alignment.add(child)
         return alignment
 
-    def set_state(self, state):
+    def update_state(self, state, reason=0):
         self._current_state = state
-        self._update_label_and_text()
+        self._update_label_and_text(reason)
 
-    def _update_label_and_text(self):
+    def _update_label_and_text(self, reason=0):
         if self._current_state == _GSM_STATE_NOT_READY:
             self._toggle_state_item.get_child().set_label('...')
             self.props.secondary_text = _('Please wait...')
 
         elif self._current_state == _GSM_STATE_DISCONNECTED:
-            self._toggle_state_item.get_child().set_label(_('Connect'))
+            if not self._failed_connection:
+                self._toggle_state_item.get_child().set_label(_('Connect'))
             self.props.secondary_text = _('Disconnected')
+            icon = Icon(icon_name='dialog-ok', \
+                            icon_size=gtk.ICON_SIZE_MENU)
+            self._toggle_state_item.set_image(icon)
 
         elif self._current_state == _GSM_STATE_CONNECTING:
             self._toggle_state_item.get_child().set_label(_('Cancel'))
             self.props.secondary_text = _('Connecting...')
+            icon = Icon(icon_name='dialog-cancel', \
+                            icon_size=gtk.ICON_SIZE_MENU)
+            self._toggle_state_item.set_image(icon)
 
         elif self._current_state == _GSM_STATE_CONNECTED:
+            self._failed_connection = False
             self._toggle_state_item.get_child().set_label(_('Disconnect'))
-            self.props.secondary_text = _('Connected')
-            
-        elif self._current_state == _GSM_STATE_NEED_AUTH:
-            self._toggle_state_item.get_child().set_label(_('Sim requires Pin/Puk'))
-            self.props.secondary_text = _('Authentication Error')
+            self.update_connection_time()
+            icon = Icon(icon_name='media-eject', \
+                            icon_size=gtk.ICON_SIZE_MENU)
+            self._toggle_state_item.set_image(icon)
             
+        elif self._current_state == _GSM_STATE_FAILED:
+            message_error = self._get_error_by_nm_reason(reason)
+            self.add_alert(message_error[0], message_error[1])
         else:
             raise ValueError('Invalid GSM state while updating label and ' \
                              'text, %s' % str(self._current_state))
@@ -300,21 +314,63 @@ class GsmPalette(Palette):
         if self._current_state == _GSM_STATE_NOT_READY:
             pass
         elif self._current_state == _GSM_STATE_DISCONNECTED:
+            self.error_title_label.hide()
+            self.error_description_label.hide()
             self.emit('gsm-connect')
         elif self._current_state == _GSM_STATE_CONNECTING:
             self.emit('gsm-disconnect')
         elif self._current_state == _GSM_STATE_CONNECTED:
             self.emit('gsm-disconnect')
-        elif self._current_state == _GSM_STATE_NEED_AUTH:
-            self.emit('gsm-disconnect')
         else:
             raise ValueError('Invalid GSM state while emitting signal, %s' % \
                              str(self._current_state))
 
+    def add_alert(self, error, suggestion):
+        self._failed_connection = True
+        self._toggle_state_item.get_child().set_label(_('Try connection again'))
+
+        title = _('Error: %s') % error
+        self.error_title_label.set_markup('<b>%s</b>' % title)
+        self.error_title_label.show()
+
+        message = _('Suggestion: %s') % suggestion
+        self.error_description_label.set_text(message)
+        self.error_description_label.show()
+
+    def update_connection_time(self, connection_time=None):
+        if connection_time is not None:
+            self.props.secondary_text = _('Connected for %s') % \
+                    connection_time.strftime('%H:%M:%S')
+        else:
+            self.props.secondary_text = _('Connected for %s') % '00:00:00'
+
+    def update_stats(self, in_bytes, out_bytes):
+        in_KBytes = in_bytes / 1024
+        out_KBytes = out_bytes / 1024
+        self._data_label_up.set_text(_("%d KB") % (out_KBytes))
+        self._data_label_down.set_text(_("%d KB") % (in_KBytes))
+
+    def _get_error_by_nm_reason(self, reason):
+        if reason in [network.NM_DEVICE_STATE_REASON_NO_SECRETS,
+                      network.NM_DEVICE_STATE_REASON_GSM_PIN_CHECK_FAILED]:
+            message = _('Check your Pin/Puk configuration.')
+        elif reason in [network.NM_DEVICE_STATE_REASON_PPP_DISCONNECT,
+                        network.NM_DEVICE_STATE_REASON_PPP_FAILED]:
+            message = _('Check your Access Point Name ' \
+                            '(APN) configuration')
+        elif reason in [network.NM_DEVICE_STATE_REASON_MODEM_NO_CARRIER,
+                        network.NM_DEVICE_STATE_REASON_MODEM_DIAL_TIMEOUT]:
+            message = _('Check the Number configuration.')
+        elif reason == network.NM_DEVICE_STATE_REASON_CONFIG_FAILED:
+            message = _('Check your configuration.')
+        else:
+            message = ''
+        message_tuple = (network.get_error_by_reason(reason), message)
+        return message_tuple
+
 
 class WirelessDeviceView(ToolButton):
 
-    _ICON_NAME = 'network-wireless'
     FRAME_POSITION_RELATIVE = 302
 
     def __init__(self, device):
@@ -333,7 +389,7 @@ class WirelessDeviceView(ToolButton):
         self._active_ap_op = None
 
         self._icon = PulsingIcon()
-        self._icon.props.icon_name = get_icon_state(self._ICON_NAME, 0)
+        self._icon.props.icon_name = get_icon_state('network-wireless', 0)
         self._inactive_color = xocolor.XoColor( \
             "%s,%s" % (style.COLOR_BUTTON_GREY.get_svg(),
                        style.COLOR_TRANSPARENT.get_svg()))
@@ -347,8 +403,6 @@ class WirelessDeviceView(ToolButton):
         self._palette = WirelessPalette(self._name)
         self._palette.connect('deactivate-connection',
                               self.__deactivate_connection_cb)
-        self._palette.connect('create-connection',
-                              self.__create_connection_cb)
         self.set_palette(self._palette)
         self._palette.set_group_id('frame')
 
@@ -418,11 +472,6 @@ class WirelessDeviceView(ToolButton):
     def __ap_properties_changed_cb(self, properties):
         self._update_properties(properties)
 
-    def _name_encodes_colors(self):
-        """Match #XXXXXX,#YYYYYY at the end of the network name"""
-        return self._name[-7] == '#' and self._name[-8] == ',' \
-            and self._name[-15] == '#'
-
     def _update_properties(self, properties):
         if 'Mode' in properties:
             self._mode = properties['Mode']
@@ -438,11 +487,9 @@ class WirelessDeviceView(ToolButton):
             self._frequency = properties['Frequency']
 
         if self._color == None:
-            if self._mode == network.NM_802_11_MODE_ADHOC \
-                    and self._name_encodes_colors():
-                encoded_color = self._name.split("#", 1)
-                if len(encoded_color) == 2:
-                    self._color = xocolor.XoColor('#' + encoded_color[1])
+            if self._mode == network.NM_802_11_MODE_ADHOC and \
+                    network.is_sugar_adhoc_network(self._name):
+                self._color = profile.get_color()
             else:
                 sha_hash = hashlib.sha1()
                 data = self._name + hex(self._flags)
@@ -467,7 +514,7 @@ class WirelessDeviceView(ToolButton):
         else:
             self._icon.props.badge_name = None
 
-        self._palette.props.primary_text = self._name
+        self._palette.props.primary_text = glib.markup_escape_text(self._name)
 
         self._update_state()
         self._update_color()
@@ -478,14 +525,24 @@ class WirelessDeviceView(ToolButton):
         else:
             state = network.DEVICE_STATE_UNKNOWN
 
-        if state == network.DEVICE_STATE_ACTIVATED:
-            icon_name = '%s-connected' % self._ICON_NAME
-        else:
-            icon_name = self._ICON_NAME
+        if self._mode != network.NM_802_11_MODE_ADHOC and \
+                network.is_sugar_adhoc_network(self._name) == False:
+            if state == network.DEVICE_STATE_ACTIVATED:
+                icon_name = '%s-connected' % 'network-wireless'
+            else:
+                icon_name = 'network-wireless'
 
-        icon_name = get_icon_state(icon_name, self._strength)
-        if icon_name:
-            self._icon.props.icon_name = icon_name
+            icon_name = get_icon_state(icon_name, self._strength)
+            if icon_name:
+                self._icon.props.icon_name = icon_name
+        else:
+            channel = network.frequency_to_channel(self._frequency)
+            if state == network.DEVICE_STATE_ACTIVATED:
+                self._icon.props.icon_name = 'network-adhoc-%s-connected' \
+                        % channel
+            else:
+                self._icon.props.icon_name = 'network-adhoc-%s' % channel
+            self._icon.props.base_color = profile.get_color()
 
         if state == network.DEVICE_STATE_PREPARE or \
            state == network.DEVICE_STATE_CONFIG or \
@@ -524,54 +581,6 @@ class WirelessDeviceView(ToolButton):
                     netmgr.DeactivateConnection(conn_o)
                     break
 
-    def __create_connection_cb(self, palette, data=None):
-        """Create an 802.11 IBSS network.
-
-        The user's color is encoded at the end of the network name. The network
-        name is truncated so that it does not exceed the 32 byte SSID limit.
-        """
-        client = gconf.client_get_default()
-        nick = client.get_string('/desktop/sugar/user/nick').decode('utf-8')
-        color = client.get_string('/desktop/sugar/user/color')
-        color_suffix = ' %s' % color
-
-        format = _('%s\'s network').encode('utf-8')
-        extra_length = (len(format) - len('%s')) + len(color_suffix)
-        name_limit = 32 - extra_length
-
-        # truncate the nick and use a regex to drop any partial characters
-        # at the end
-        nick = nick.encode('utf-8')[:name_limit]
-        pattern = "([\xf6-\xf7][\x80-\xbf]{0,2}|[\xe0-\xef][\x80-\xbf]{0,1}|[\xc0-\xdf])$"
-        nick = re.sub(pattern, '', nick)
-
-        connection_name = format % nick
-        connection_name += color_suffix
-
-        connection = network.find_connection_by_ssid(connection_name)
-        if connection is None:
-            settings = Settings()
-            settings.connection.id = 'Auto ' + connection_name
-            uuid = settings.connection.uuid = unique_id()
-            settings.connection.type = '802-11-wireless'
-            settings.wireless.ssid = dbus.ByteArray(connection_name)
-            settings.wireless.band = 'bg'
-            settings.wireless.mode = 'adhoc'
-            settings.ip4_config = IP4Config()
-            settings.ip4_config.method = 'link-local'
-
-            connection = network.add_connection(uuid, settings)
-
-        obj = self._bus.get_object(_NM_SERVICE, _NM_PATH)
-        netmgr = dbus.Interface(obj, _NM_IFACE)
-
-        netmgr.ActivateConnection(network.SETTINGS_SERVICE,
-                                  connection.path,
-                                  self._device.object_path,
-                                  '/',
-                                  reply_handler=self.__activate_reply_cb,
-                                  error_handler=self.__activate_error_cb)
-
     def __activate_reply_cb(self, connection):
         logging.debug('Network created: %s', connection)
 
@@ -583,7 +592,7 @@ class OlpcMeshDeviceView(ToolButton):
     _ICON_NAME = 'network-mesh'
     FRAME_POSITION_RELATIVE = 302
 
-    def __init__(self, device):
+    def __init__(self, device, state):
         ToolButton.__init__(self)
 
         self._bus = dbus.SystemBus()
@@ -593,10 +602,11 @@ class OlpcMeshDeviceView(ToolButton):
         self._channel = 0
 
         self._icon = PulsingIcon(icon_name=self._ICON_NAME)
-        self._icon.props.pulse_color = xocolor.XoColor( \
+        self._inactive_color = xocolor.XoColor( \
             "%s,%s" % (style.COLOR_BUTTON_GREY.get_svg(),
                        style.COLOR_TRANSPARENT.get_svg()))
-        self._icon.props.base_color = profile.get_color()
+        self._icon.props.pulse_color = profile.get_color()
+        self._icon.props.base_color = self._inactive_color
 
         self.set_icon_widget(self._icon)
         self._icon.show()
@@ -608,42 +618,25 @@ class OlpcMeshDeviceView(ToolButton):
         self.set_palette(self._palette)
         self._palette.set_group_id('frame')
 
+        self.update_state(state)
+
         self._device_props = dbus.Interface(self._device,
                                             'org.freedesktop.DBus.Properties')
-        self._device_props.GetAll(_NM_DEVICE_IFACE, byte_arrays=True,
-                              reply_handler=self.__get_device_props_reply_cb,
-                              error_handler=self.__get_device_props_error_cb)
         self._device_props.Get(_NM_OLPC_MESH_IFACE, 'ActiveChannel',
                             reply_handler=self.__get_active_channel_reply_cb,
                             error_handler=self.__get_active_channel_error_cb)
 
-        self._bus.add_signal_receiver(self.__state_changed_cb,
-                                      signal_name='StateChanged',
-                                      path=self._device.object_path,
-                                      dbus_interface=_NM_DEVICE_IFACE)
         self._bus.add_signal_receiver(self.__wireless_properties_changed_cb,
                                       signal_name='PropertiesChanged',
                                       path=device.object_path,
                                       dbus_interface=_NM_OLPC_MESH_IFACE)
 
     def disconnect(self):
-        self._bus.remove_signal_receiver(self.__state_changed_cb,
-                                         signal_name='StateChanged',
-                                         path=self._device.object_path,
-                                         dbus_interface=_NM_DEVICE_IFACE)
         self._bus.remove_signal_receiver(self.__wireless_properties_changed_cb,
                                          signal_name='PropertiesChanged',
                                          path=self._device.object_path,
                                          dbus_interface=_NM_OLPC_MESH_IFACE)
 
-    def __get_device_props_reply_cb(self, properties):
-        if 'State' in properties:
-            self._device_state = properties['State']
-            self._update()
-
-    def __get_device_props_error_cb(self, err):
-        logging.error('Error getting the device properties: %s', err)
-
     def __get_active_channel_reply_cb(self, channel):
         self._channel = channel
         self._update_text()
@@ -661,7 +654,8 @@ class OlpcMeshDeviceView(ToolButton):
             self._update_text()
 
     def _update_text(self):
-        text = _("Mesh Network") + " " + str(self._channel)
+        channel = str(self._channel)
+        text = _("Mesh Network %s") % glib.markup_escape_text(channel)
         self._palette.props.primary_text = text
 
     def _update(self):
@@ -671,12 +665,20 @@ class OlpcMeshDeviceView(ToolButton):
                      network.DEVICE_STATE_CONFIG,
                      network.DEVICE_STATE_NEED_AUTH,
                      network.DEVICE_STATE_IP_CONFIG]:
+            self._icon.props.base_color = self._inactive_color
+            self._icon.props.pulse_color = profile.get_color()
             self._palette.set_connecting()
             self._icon.props.pulsing = True
         elif state == network.DEVICE_STATE_ACTIVATED:
             address = self._device_props.Get(_NM_DEVICE_IFACE, 'Ip4Address')
             self._palette.set_connected_with_channel(self._channel, address)
+            self._icon.props.base_color = profile.get_color()
             self._icon.props.pulsing = False
+        self._update_text()
+
+    def update_state(self, state):
+        self._device_state = state
+        self._update()
 
     def __deactivate_connection(self, palette, data=None):
         obj = self._bus.get_object(_NM_SERVICE, _NM_PATH)
@@ -774,6 +776,10 @@ class GsmDeviceView(TrayIcon):
                                         '/',
                                         reply_handler=self.__connect_cb,
                                         error_handler=self.__connect_error_cb)
+        else:
+            self._palette.add_alert(_('No GSM connection available.'), \
+                                        _('Create a connection in the ' \
+                                              'control panel.'))
 
     def __connect_cb(self, active_connection):
         logging.debug('Connected successfully to gsm device, %s',
@@ -806,16 +812,17 @@ class GsmDeviceView(TrayIcon):
         raise RuntimeError('Error when disconnecting gsm device, %s' % error)
 
     def __state_changed_cb(self, new_state, old_state, reason):
-        logging.debug('State: %s to %s, reason %s', old_state, new_state, reason)
-        self._update_state(int(new_state))
+        logging.debug('State: %s to %s, reason %s', old_state,
+                      new_state, reason)
+        self._update_state(int(new_state), int(old_state), int(reason))
 
     def __current_state_check_cb(self, properties):
-        self._update_state(int(properties['State']))
+        self._update_state(int(properties['State']), 0, 0)
 
     def __current_state_check_error_cb(self, error):
         raise RuntimeError('Error when checking gsm device state, %s' % error)
 
-    def _update_state(self, state):
+    def _update_state(self, state, old_state, reason):
         gsm_state = None
 
         if state is network.DEVICE_STATE_ACTIVATED:
@@ -823,20 +830,22 @@ class GsmDeviceView(TrayIcon):
             connection = network.find_gsm_connection()
             if connection is not None:
                 connection.set_connected()
-                self._connection_timestamp =  time.time() - \
+                self._connection_timestamp = time.time() - \
                         connection.get_settings().connection.timestamp
                 self._connection_time_handler = gobject.timeout_add_seconds( \
                         1, self.__connection_timecount_cb)
-                self._update_stats(0, 0)
-                self._update_connection_time()                
-                self._palette.info_box.show() 
+                self._palette.update_connection_time()
+                self._palette.update_stats(0, 0)
+                if self._palette is not None:
+                    self._palette.connection_info_box.show()
 
         elif state is network.DEVICE_STATE_DISCONNECTED:
             gsm_state = _GSM_STATE_DISCONNECTED
             self._connection_timestamp = 0
             if self._connection_time_handler is not None:
                 gobject.source_remove(self._connection_time_handler)
-            self._palette.info_box.hide() 
+            if self._palette is not None:
+                self._palette.connection_info_box.hide()
 
         elif state in [network.DEVICE_STATE_UNMANAGED,
                        network.DEVICE_STATE_UNAVAILABLE,
@@ -845,14 +854,15 @@ class GsmDeviceView(TrayIcon):
 
         elif state in [network.DEVICE_STATE_PREPARE,
                        network.DEVICE_STATE_CONFIG,
-                       network.DEVICE_STATE_IP_CONFIG]:
+                       network.DEVICE_STATE_IP_CONFIG,
+                       network.DEVICE_STATE_NEED_AUTH]:
             gsm_state = _GSM_STATE_CONNECTING
-            
-        elif state in [network.DEVICE_STATE_NEED_AUTH]:
-            gsm_state = _GSM_STATE_NEED_AUTH
-            
+
+        elif state == network.DEVICE_STATE_FAILED:
+            gsm_state = _GSM_STATE_FAILED
+
         if self._palette is not None:
-            self._palette.set_state(gsm_state)
+            self._palette.update_state(gsm_state, reason)
 
     def disconnect(self):
         self._bus.remove_signal_receiver(self.__state_changed_cb,
@@ -861,38 +871,22 @@ class GsmDeviceView(TrayIcon):
                                          dbus_interface=_NM_DEVICE_IFACE)
 
     def __ppp_stats_changed_cb(self, in_bytes, out_bytes):
-        self._update_stats(in_bytes, out_bytes)
-
-    def _update_stats(self, in_bytes, out_bytes):
-        in_KBytes = in_bytes / 1024
-        out_KBytes = out_bytes / 1024
-        text = _("Data sent %d KB / received %d KB") % (out_KBytes, in_KBytes)
-        self._palette.data_label.set_text(text)
+        self._palette.update_stats(in_bytes, out_bytes)
 
     def __connection_timecount_cb(self):
         self._connection_timestamp = self._connection_timestamp + 1
-        self._update_connection_time()
+        connection_time = \
+            datetime.datetime.fromtimestamp(self._connection_timestamp)
+        self._palette.update_connection_time(connection_time)
         return True
 
-    def _update_connection_time(self):
-        connection_time = datetime.datetime.fromtimestamp( \
-                self._connection_timestamp)
-        text = _("Connection time ") + connection_time.strftime('%H : %M : %S')
-        self._palette.connection_time_label.set_text(text)
 
 class WirelessDeviceObserver(object):
-    def __init__(self, device, tray, device_type):
+    def __init__(self, device, tray):
         self._device = device
         self._device_view = None
         self._tray = tray
-
-        if device_type == network.DEVICE_TYPE_802_11_WIRELESS:
-            self._device_view = WirelessDeviceView(self._device)
-        elif device_type == network.DEVICE_TYPE_802_11_OLPC_MESH:
-            self._device_view = OlpcMeshDeviceView(self._device)
-        else:
-            raise ValueError('Unimplemented device type %d' % device_type)
-
+        self._device_view = WirelessDeviceView(self._device)
         self._tray.add_device(self._device_view)
 
     def disconnect(self):
@@ -902,6 +896,63 @@ class WirelessDeviceObserver(object):
         self._device_view = None
 
 
+class MeshDeviceObserver(object):
+    def __init__(self, device, tray):
+        self._bus = dbus.SystemBus()
+        self._device = device
+        self._device_view = None
+        self._tray = tray
+
+        props = dbus.Interface(self._device, dbus.PROPERTIES_IFACE)
+        props.GetAll(_NM_DEVICE_IFACE, byte_arrays=True,
+                     reply_handler=self.__get_device_props_reply_cb,
+                     error_handler=self.__get_device_props_error_cb)
+
+        self._bus.add_signal_receiver(self.__state_changed_cb,
+                                      signal_name='StateChanged',
+                                      path=self._device.object_path,
+                                      dbus_interface=_NM_DEVICE_IFACE)
+
+    def _remove_device_view(self):
+        self._device_view.disconnect()
+        self._tray.remove_device(self._device_view)
+        self._device_view = None
+
+    def disconnect(self):
+        if self._device_view is not None:
+            self._remove_device_view()
+
+        self._bus.remove_signal_receiver(self.__state_changed_cb,
+                                         signal_name='StateChanged',
+                                         path=self._device.object_path,
+                                         dbus_interface=_NM_DEVICE_IFACE)
+
+    def __get_device_props_reply_cb(self, properties):
+        if 'State' in properties:
+            self._update_state(properties['State'])
+
+    def __get_device_props_error_cb(self, err):
+        logging.error('Error getting the device properties: %s', err)
+
+    def __state_changed_cb(self, new_state, old_state, reason):
+        self._update_state(new_state)
+
+    def _update_state(self, state):
+        if state in (network.DEVICE_STATE_PREPARE, network.DEVICE_STATE_CONFIG,
+                     network.DEVICE_STATE_NEED_AUTH,
+                     network.DEVICE_STATE_IP_CONFIG,
+                     network.DEVICE_STATE_ACTIVATED):
+            if self._device_view is not None:
+                self._device_view.update_state(state)
+                return
+
+            self._device_view = OlpcMeshDeviceView(self._device, state)
+            self._tray.add_device(self._device_view)
+        else:
+            if self._device_view is not None:
+                self._remove_device_view()
+
+
 class WiredDeviceObserver(object):
     def __init__(self, device, tray):
         self._bus = dbus.SystemBus()
@@ -1003,9 +1054,11 @@ class NetworkManagerObserver(object):
         if device_type == network.DEVICE_TYPE_802_3_ETHERNET:
             device = WiredDeviceObserver(nm_device, self._tray)
             self._devices[device_op] = device
-        elif device_type in [network.DEVICE_TYPE_802_11_WIRELESS,
-                             network.DEVICE_TYPE_802_11_OLPC_MESH]:
-            device = WirelessDeviceObserver(nm_device, self._tray, device_type)
+        elif device_type == network.DEVICE_TYPE_802_11_WIRELESS:
+            device = WirelessDeviceObserver(nm_device, self._tray)
+            self._devices[device_op] = device
+        elif device_type == network.DEVICE_TYPE_802_11_OLPC_MESH:
+            device = MeshDeviceObserver(nm_device, self._tray)
             self._devices[device_op] = device
         elif device_type == network.DEVICE_TYPE_GSM_MODEM:
             device = GsmDeviceObserver(nm_device, self._tray)
