@@ -36,6 +36,7 @@ from sugar.activity.activityhandle import ActivityHandle
 from jarabe.model import bundleregistry
 from jarabe.view.palettes import ActivityPalette
 from jarabe.view import launcher
+from jarabe.journal import misc
 
 class ActivitiesTreeView(gtk.TreeView):
     __gtype_name__ = 'SugarActivitiesTreeView'
@@ -143,13 +144,7 @@ class ActivitiesTreeView(gtk.TreeView):
         registry = bundleregistry.get_registry()
         bundle = registry.get_bundle(row[ListModel.COLUMN_BUNDLE_ID])
 
-        activity_id = activityfactory.create_activity_id()
-
-        client = gconf.client_get_default()
-        xo_color = XoColor(client.get_string('/desktop/sugar/user/color'))
-
-        launcher.add_launcher(activity_id, bundle.get_icon(), xo_color)
-        activityfactory.create(bundle, ActivityHandle(activity_id))
+        misc.launch(bundle)
 
     def set_filter(self, query):
         self._query = query.lower()
@@ -376,7 +371,7 @@ class ActivitiesList(gtk.VBox):
         if response_id == gtk.RESPONSE_OK:
             registry = bundleregistry.get_registry()
             bundle = registry.get_bundle(bundle_id)
-            registry.uninstall(bundle)
+            registry.uninstall(bundle, delete_profile=True)
 
 class ActivityListPalette(ActivityPalette):
     __gtype_name__ = 'SugarActivityListPalette'
@@ -406,13 +401,7 @@ class ActivityListPalette(ActivityPalette):
         self._favorite_item.show()
 
         if activity_info.is_user_activity():
-            menu_item = MenuItem(_('Erase'), 'list-remove')
-            menu_item.connect('activate', self.__erase_activate_cb)
-            self.menu.append(menu_item)
-            menu_item.show()
-
-            if not os.access(activity_info.get_path(), os.W_OK):
-                menu_item.props.sensitive = False
+            self._add_erase_option(registry, activity_info)
 
         registry = bundleregistry.get_registry()
         self._activity_changed_sid = registry.connect('bundle_changed',
@@ -421,6 +410,16 @@ class ActivityListPalette(ActivityPalette):
 
         self.connect('destroy', self.__destroy_cb)
 
+    def _add_erase_option(self, registry, activity_info):
+        menu_item = MenuItem(_('Erase'), 'list-remove')
+        menu_item.connect('activate', self.__erase_activate_cb)
+        self.menu.append(menu_item)
+        menu_item.show()
+
+        if not os.access(activity_info.get_path(), os.W_OK) or \
+            registry.is_activity_protected(self._bundle_id):
+                menu_item.props.sensitive = False
+
     def __destroy_cb(self, palette):
         self.disconnect(self._activity_changed_sid)
 
