@@ -19,6 +19,7 @@ import os
 import logging
 import traceback
 
+import gconf
 import gobject
 import gio
 import simplejson
@@ -66,6 +67,14 @@ class BundleRegistry(gobject.GObject):
         self._last_defaults_mtime = -1
         self._favorite_bundles = {}
 
+        client = gconf.client_get_default()
+        self._protected_activities = client.get_list(
+                                    '/desktop/sugar/protected_activities',
+                                     gconf.VALUE_STRING)
+
+        if self._protected_activities is None:
+            self._protected_activities = []
+
         try:
             self._load_favorites()
         except Exception:
@@ -159,12 +168,22 @@ class BundleRegistry(gobject.GObject):
         self._write_favorites_file()
 
     def get_bundle(self, bundle_id):
-        """Returns an bundle given his service name"""
+        """Returns a bundle given service name or substring,
+        returns None if there is either no match, or more than one
+        match by substring."""
+        result = []
+        key = bundle_id.lower()
+
         for bundle in self._bundles:
-            if bundle.get_bundle_id() == bundle_id:
+            name = bundle.get_bundle_id()
+            if name == bundle_id:
                 return bundle
+            if key in name.lower():
+                result.append(bundle)
+        if len(result) == 1:
+            return result[0]
         return None
-    
+
     def __iter__(self):
         return self._bundles.__iter__()
 
@@ -302,6 +321,9 @@ class BundleRegistry(gobject.GObject):
         key = self._get_favorite_key(bundle_id, version)
         return key in self._favorite_bundles
 
+    def is_activity_protected(self, bundle_id):
+        return bundle_id in self._protected_activities
+
     def set_bundle_position(self, bundle_id, version, x, y):
         key = self._get_favorite_key(bundle_id, version)
         if key not in self._favorite_bundles:
@@ -376,7 +398,7 @@ class BundleRegistry(gobject.GObject):
         elif not self.add_bundle(install_path):
             raise RegistrationException
 
-    def uninstall(self, bundle, force=False):        
+    def uninstall(self, bundle, force=False, delete_profile=False):
         # TODO treat ContentBundle in special way
         # needs rethinking while fixing ContentBundle support
         if isinstance(bundle, ContentBundle) or \
@@ -399,7 +421,7 @@ class BundleRegistry(gobject.GObject):
 
         install_path = act.get_path()
 
-        bundle.uninstall(install_path, force)
+        bundle.uninstall(install_path, force, delete_profile)
 
         if not self.remove_bundle(install_path):
             raise RegistrationException
