@@ -20,6 +20,7 @@ import subprocess
 import sys
 import time
 from optparse import OptionParser
+from gettext import gettext as _
 
 import gtk
 import gobject
@@ -36,6 +37,7 @@ def _run_xephyr(display, dpi, dimensions, fullscreen):
     cmd = [ 'Xephyr' ]
     cmd.append(':%d' % display)
     cmd.append('-ac') 
+    cmd += ['-title', _('Sugar in a window')]
 
     screen_size = (gtk.gdk.screen_width(), gtk.gdk.screen_height())
 
@@ -122,10 +124,14 @@ def _setup_env(display, scaling, emulator_pid):
             env.get_profile_path(), 'logs', 'telepathy-gabble.log')
     os.environ['SALUT_LOGFILE'] = os.path.join(
             env.get_profile_path(), 'logs', 'telepathy-salut.log')
+    os.environ['MC_LOGFILE'] = os.path.join(
+            env.get_profile_path(), 'logs', 'mission-control.log')
     os.environ['STREAM_ENGINE_LOGFILE'] = os.path.join(
             env.get_profile_path(), 'logs', 'telepathy-stream-engine.log')
     os.environ['DISPLAY'] = ":%d" % (display)
     os.environ['SUGAR_EMULATOR_PID'] = emulator_pid
+    os.environ['MC_ACCOUNT_DIR'] = os.path.join(
+            env.get_profile_path(), 'accounts')
 
     if scaling:
         os.environ['SUGAR_SCALING'] = scaling
